<?php

namespace Moblyze\Forms\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Moblyze\Forms\Http\Requests\FormsRequest;
use Moblyze\Forms\Interfaces\FormsRepositoryInterface;
use Moblyze\Forms\Models\Forms;

/**
 * Resource controller class for forms.
 */
class FormsResourceController extends BaseController
{

    /**
     * Initialize forms resource controller.
     *
     * @param type FormsRepositoryInterface $forms
     *
     * @return null
     */
    public function __construct(FormsRepositoryInterface $forms)
    {
        parent::__construct();
        $this->repository = $forms;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Moblyze\Forms\Repositories\Criteria\FormsResourceCriteria::class);
    }

    /**
     * Display a list of forms.
     *
     * @return Response
     */
    public function index(FormsRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Moblyze\Forms\Repositories\Presenter\FormsPresenter::class)
                ->$function();
        }

        $forms = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('forms::forms.names'))
            ->view('forms::forms.index', true)
            ->data(compact('forms', 'view'))
            ->output();
    }

    /**
     * Display forms.
     *
     * @param Request $request
     * @param Model   $forms
     *
     * @return Response
     */
    public function show(FormsRequest $request, Forms $forms)
    {

        if ($forms->exists) {
            $view = 'forms::forms.show';
        } else {
            $view = 'forms::forms.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('forms::forms.name'))
            ->data(compact('forms'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new forms.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(FormsRequest $request)
    {

        $forms = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('forms::forms.name')) 
            ->view('forms::forms.create', true) 
            ->data(compact('forms'))
            ->output();
    }

    /**
     * Create new forms.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(FormsRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $forms                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('forms::forms.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('forms/forms/' . $forms->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/forms/forms'))
                ->redirect();
        }

    }

    /**
     * Show forms for editing.
     *
     * @param Request $request
     * @param Model   $forms
     *
     * @return Response
     */
    public function edit(FormsRequest $request, Forms $forms)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('forms::forms.name'))
            ->view('forms::forms.edit', true)
            ->data(compact('forms'))
            ->output();
    }

    /**
     * Update the forms.
     *
     * @param Request $request
     * @param Model   $forms
     *
     * @return Response
     */
    public function update(FormsRequest $request, Forms $forms)
    {
        try {
            $attributes = $request->all();

            $forms->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('forms::forms.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('forms/forms/' . $forms->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('forms/forms/' . $forms->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the forms.
     *
     * @param Model   $forms
     *
     * @return Response
     */
    public function destroy(FormsRequest $request, Forms $forms)
    {
        try {

            $forms->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('forms::forms.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('forms/forms/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('forms/forms/' . $forms->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple forms.
     *
     * @param Model   $forms
     *
     * @return Response
     */
    public function delete(FormsRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('forms::forms.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('forms/forms'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/forms/forms'))
                ->redirect();
        }

    }

    /**
     * Restore deleted forms.
     *
     * @param Model   $forms
     *
     * @return Response
     */
    public function restore(FormsRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('forms::forms.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/forms/forms'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/forms/forms/'))
                ->redirect();
        }

    }

}
